@@ -1,6 +1,7 @@
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
 # Copyright 2018 - Extreme Networks, Inc.
+# Copyright 2019 - Nokia Networks
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -78,7 +79,7 @@ api_opts = [
     cfg.BoolOpt(
         'enable_ssl_api',
         default=False,
-        help=_('Enable the integrated stand-alone API to service requests'
+        help=_('Enable the integrated stand-alone API to service requests '
                'via HTTPS instead of HTTP.')
     ),
     cfg.IntOpt(
@@ -86,6 +87,17 @@ api_opts = [
         help=_('Number of workers for Mistral API service '
                'default is equal to the number of CPUs available if that can '
                'be determined, else a default worker count of 1 is returned.')
+    ),
+    cfg.StrOpt(
+        'validation_mode',
+        default='mandatory',
+        choices=['enabled', 'mandatory', 'disabled'],
+        help=_("Defines in what cases Mistral will be validating the syntax "
+               "of workflow YAML definitions. If 'enabled' is set the service "
+               "will be validating the syntax but only if it's not explicitly "
+               "turned off in the API request. 'disabled' disables validation "
+               "for all API requests. 'mandatory' enables validation for all "
+               "API requests.")
     )
 ]
 
@@ -184,11 +196,29 @@ engine_opts = [
                ' set to a negative value Mistral will never be doing '
                ' this check.')
     ),
+    cfg.IntOpt(
+        'execution_integrity_check_batch_size',
+        default=5,
+        min=1,
+        help=_('A number of task executions in RUNNING state that the'
+               ' execution integrity checker can process in a single'
+               ' iteration.')
+    ),
     cfg.IntOpt(
         'action_definition_cache_time',
         default=60,
         help=_('A number of seconds that indicates how long action '
                'definitions should be stored in the local cache.')
+    ),
+    cfg.BoolOpt(
+        'start_subworkflows_via_rpc',
+        default=False,
+        help=(
+            'Enables starting subworkflows via RPC. Use "False" to start '
+            'subworkflow within the same engine instance. Use "True" '
+            'to start subworkflow via RPC to improve load balancing '
+            'in case of several engine instances.'
+        )
     )
 ]
 
@@ -206,9 +236,13 @@ executor_opts = [
     cfg.HostAddressOpt(
         'host',
         default='0.0.0.0',
-        help=_('Name of the executor node. This can be an opaque '
-               'identifier. It is not necessarily a hostname, '
-               'FQDN, or IP address.')
+        help=_('Name of the executor node. This can be an any string '
+               'name/identifier. It is not necessarily a hostname, '
+               'FQDN, or IP address. It is also related to the "target" '
+               'attribute of tasks defined in a workflow text. If "target" '
+               'is defined for a task then the action of the task will be '
+               'sent to one of the executors that have the same value in the '
+               '"host" property.')
     ),
     cfg.StrOpt(
         'topic',
@@ -222,6 +256,13 @@ executor_opts = [
     )
 ]
 
+scheduler_type_opt = cfg.StrOpt(
+    'scheduler_type',
+    default='legacy',
+    choices=['legacy', 'default'],
+    help=_('The name of the scheduler implementation used in the system.')
+)
+
 scheduler_opts = [
     cfg.FloatOpt(
         'fixed_delay',
@@ -255,6 +296,34 @@ scheduler_opts = [
             'If this property equals None then there is no '
             'restriction on selection.'
         )
+    ),
+    cfg.FloatOpt(
+        'captured_job_timeout',
+        default=30,
+        min=1,
+        help=(
+            'Defines how soon (in seconds) a scheduled job captured for '
+            'processing becomes eligible for capturing by other schedulers '
+            'again. This option is needed to prevent situations when a '
+            'scheduler instance captured a job and failed while processing '
+            'and so this job can never be processed again because it is '
+            'marked as captured.'
+        )
+    ),
+    cfg.FloatOpt(
+        'pickup_job_after',
+        default=60,
+        min=1,
+        help='Time period given to a scheduler to process a scheduled job '
+             'locally before it becomes eligible for processing by other '
+             'scheduler instances.'
+             'For example, a job needs to run at 12:00:00. When a scheduler '
+             'starts processing it it has 60 seconds (or other configured '
+             'value) to complete the job. If the scheduler did not complete '
+             'the job within this period it most likely means that the '
+             'scheduler process crashed. In this case another scheduler '
+             'instance will pick it up from the Job Store, but not earlier '
+             'than 12:01:00 and try to process it.'
     )
 ]
 
@@ -385,19 +454,27 @@ action_heartbeat_opts = [
         min=0,
         default=15,
         help=_('The maximum amount of missed heartbeats to be allowed. '
-               'If set to 0 then this feature won\'t be enabled. '
+               'If set to 0 then this feature is disabled. '
                'See check_interval for more details.')
     ),
     cfg.IntOpt(
         'check_interval',
         min=0,
         default=20,
-        help=_('How often the action executions are checked (in seconds). '
-               'For example when check_interval = 10, check action '
+        help=_('How often (in seconds) action executions are checked. '
+               'For example when check_interval is 10, check action '
                'executions every 10 seconds. When the checker runs it will '
                'transit all running action executions to error if the last '
                'heartbeat received is older than 10 * max_missed_heartbeats '
-               'seconds. If set to 0 then this feature won\'t be enabled.')
+               'seconds. If set to 0 then this feature is disabled.')
+    ),
+    cfg.IntOpt(
+        'batch_size',
+        min=0,
+        default=10,
+        help=_('The maximum number of action executions processed during '
+               'one iteration of action execution heartbeat checker. If set '
+               'to 0 then there is no limit.')
     ),
     cfg.IntOpt(
         'first_heartbeat_timeout',
@@ -516,6 +593,23 @@ yaql_opts = [
         help=_('The memory usage quota (in bytes) for all data produced by '
                'the expression (or any part of it). -1 means no limitation.')
     ),
+    cfg.BoolOpt(
+        'convert_input_data',
+        default=True,
+        help=_('Enables input data conversion for YAQL expressions. If set '
+               'to True then YAQL will convert mutable data structures '
+               '(lists, dicts, sets) into their immutable versions. That '
+               'will allow them to work with some constructs that require '
+               'hashable types even if elements are not hashable. For '
+               'example, it will be possible to put dicts into a set. '
+               'Although it conflicts with the base principles of such '
+               'collections (e.g. we cannot put a non-hashable type into '
+               'a set just because otherwise it will not work correctly) the '
+               'YAQL library itself allows this. '
+               'Disabling input data conversion may give significant '
+               'performance boost if the input data for an expression is '
+               'large.')
+    ),
     cfg.BoolOpt(
         'convert_tuples_to_lists',
         default=True,
@@ -572,6 +666,7 @@ KEYSTONE_GROUP = "keystone"
 
 CONF.register_opt(wf_trace_log_name_opt)
 CONF.register_opt(auth_type_opt)
+CONF.register_opt(scheduler_type_opt)
 CONF.register_opt(js_impl_opt)
 CONF.register_opt(rpc_impl_opt)
 CONF.register_opt(rpc_response_timeout_opt)
@@ -611,6 +706,7 @@ default_group_opts = itertools.chain(
     [
         wf_trace_log_name_opt,
         auth_type_opt,
+        scheduler_type_opt,
         js_impl_opt,
         rpc_impl_opt,
         rpc_response_timeout_opt,
