@@ -18,13 +18,14 @@ import datetime
 
 import mock
 import sqlalchemy as sa
+import yaml
 
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral.tests.unit.api import base
 from mistral.tests.unit import base as unit_base
-from mistral import utils
+from mistral_lib import utils
 
 WF_DEFINITION = """
 ---
@@ -179,6 +180,60 @@ wf2:
       action: std.echo output="Mistral"
 """
 
+WFS_YAML = yaml.load(WFS_DEFINITION)
+FIRST_WF_DEF = yaml.dump({
+    'version': '2.0',
+    'wf1': WFS_YAML['wf1']
+})
+SECOND_WF_DEF = yaml.dump({
+    'version': '2.0',
+    'wf2': WFS_YAML['wf2']
+})
+
+FIRST_WF_DICT = {
+    'name': 'wf1',
+    'tasks': {
+        'task1': {
+            'action': 'std.echo output="Hello"',
+            'name': 'task1',
+            'type': 'direct',
+            'version': '2.0'
+        }
+    },
+    'version': '2.0'
+}
+FIRST_WF = {
+    'name': 'wf1',
+    'tags': [],
+    'definition': FIRST_WF_DEF,
+    'spec': FIRST_WF_DICT,
+    'scope': 'private',
+    'namespace': '',
+    'is_system': False
+}
+
+SECOND_WF_DICT = {
+    'name': 'wf2',
+    'tasks': {
+        'task1': {
+            'action': 'std.echo output="Mistral"',
+            'name': 'task1',
+            'type': 'direct',
+            'version': '2.0'
+        }
+    },
+    'version': '2.0'
+}
+SECOND_WF = {
+    'name': 'wf2',
+    'tags': [],
+    'definition': SECOND_WF_DEF,
+    'spec': SECOND_WF_DICT,
+    'scope': 'private',
+    'namespace': '',
+    'is_system': False
+}
+
 MOCK_WF = mock.MagicMock(return_value=WF_DB)
 MOCK_WF_SYSTEM = mock.MagicMock(return_value=WF_DB_SYSTEM)
 MOCK_WF_WITH_INPUT = mock.MagicMock(return_value=WF_DB_WITH_INPUT)
@@ -256,7 +311,8 @@ class TestWorkflowsController(base.APITest):
             UPDATED_WF_DEFINITION,
             scope='private',
             identifier='123e4567-e89b-12d3-a456-426655440000',
-            namespace=''
+            namespace='',
+            validate=True
         )
         self.assertDictEqual(UPDATED_WF, resp.json)
 
@@ -343,6 +399,33 @@ class TestWorkflowsController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    @mock.patch.object(
+        db_api, "update_workflow_definition", MOCK_UPDATED_WF
+    )
+    def test_put_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.put(
+            '/v2/workflows?skip_validation',
+            WF_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(200, resp.status_int)
+
+    @mock.patch.object(db_api, "update_workflow_definition")
+    def test_put_multiple(self, mock_mtd):
+        self.app.put(
+            '/v2/workflows',
+            WFS_DEFINITION,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(2, mock_mtd.call_count)
+        mock_mtd.assert_any_call('wf1', FIRST_WF)
+        mock_mtd.assert_any_call('wf2', SECOND_WF)
+
     def test_put_more_workflows_with_uuid(self):
         resp = self.app.put(
             '/v2/workflows/123e4567-e89b-12d3-a456-426655440000',
@@ -414,6 +497,18 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual(409, resp.status_int)
 
+    @mock.patch.object(db_api, "create_workflow_definition")
+    def test_post_multiple(self, mock_mtd):
+        self.app.post(
+            '/v2/workflows',
+            WFS_DEFINITION,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(2, mock_mtd.call_count)
+        mock_mtd.assert_any_call(FIRST_WF)
+        mock_mtd.assert_any_call(SECOND_WF)
+
     def test_post_invalid(self):
         resp = self.app.post(
             '/v2/workflows',
@@ -425,6 +520,18 @@ class TestWorkflowsController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    def test_post_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.post(
+            '/v2/workflows?skip_validation',
+            WF_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(201, resp.status_int)
+
     @mock.patch.object(db_api, "delete_workflow_definition", MOCK_DELETE)
     @mock.patch.object(db_api, "get_workflow_definition", MOCK_WF)
     def test_delete(self):
@@ -624,10 +731,9 @@ class TestWorkflowsController(base.APITest):
 
         self.assertEqual(400, resp.status_int)
 
-        self.assertIn(
-            "nonexist are invalid",
-            resp.body.decode()
-        )
+        response_msg = resp.body.decode()
+        self.assertIn("nonexist", response_msg)
+        self.assertIn("do not exist", response_msg)
 
     def test_validate(self):
         resp = self.app.post(
@@ -726,7 +832,8 @@ class TestWorkflowsController(base.APITest):
             WF_DEFINITION,
             scope='private',
             identifier=id_,
-            namespace='abc'
+            namespace='abc',
+            validate=True
         )
         self.assertDictEqual(WF_WITH_NAMESPACE, resp.json)
 
